--[[
    Dota Kill Sound by Crafter_z. v1.0 28/02/08
    Creado para los viciosos de la UCI que juegan Dota y WoW.
    Basado en MortalKombat .

    Unofficial Update version by Klaimmore 01/11/08
--]]

-- [Saved Variables]
DotaKillSoundUU_conf_focus = "ENABLED";
DotaKillSoundUU_killCount = 0;
DotaKillSoundUU_record = 0;
-- [/Saved Variables]

local flaw = true;

local MusicPlayTime = 0; -- Play time of the current song
local lastMusicTime = 0; -- The last time the song started playing

local lastKillTime = 0; -- The time of the last kill
local killTime = 48; -- Maximum time elapsed between kills to award multi kills
local multiKill = 0; -- Current Multikill count

local finishHim = false; -- True if the current target dropped below 20% health
local soundPath = "Interface\\AddOns\\DotaKillSoundUU\\Sounds\\";

local player_name = UnitName("player"); -- the player name
local last_targets = { }; -- last attacks
local last_sources = { }; -- last enemies against us
local BUFFER = 20; -- Size of buffer used to save targets and sources
local last_deathtime = 0; -- the last time we died

local UpdateInterval = 0.5; -- The addon update interval, in order to check the sounds currently playing
local TimeElapsed_SinceUpdate = 0; -- The time of the last sound verification
local remainingTime_CurrentSound = 0; -- The time remaining of the currently playing sound

local SoundInfo = {}; -- Info Table of the Sound Files
local SoundCatalog = {}; -- Info table of the Sizes and Head sounds per type

local sound_queue = {}; -- Queue of the sounds to be reproduced in order
local head_sound = 0;
local tail_sound = -1;
local SOUNDBUFFER = 40; -- Size of buffer used to save the list of sounds

local _;

function DotaKillSoundUU_OnLoad(self)
    DEFAULT_CHAT_FRAME:AddMessage("DotaKillSound Unofficial Update v1.4",.7,.7,1);
    DEFAULT_CHAT_FRAME:AddMessage("(Author: Klaimmore, Former Author: Crafter_z)",.7,.7,1);

    self:RegisterEvent("UNIT_HEALTH");
    self:RegisterEvent("PLAYER_TARGET_CHANGED");
    self:RegisterEvent("PLAYER_REGEN_ENABLED");
    self:RegisterEvent("PLAYER_DEAD");
    self:RegisterEvent("PLAYER_REGEN_DISABLED");
    self:RegisterEvent("UNIT_COMBAT");
    self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");

    last_targets['name'] = {};
    last_targets['time'] = {};
    last_targets['hit'] = {};
    last_targets['crit'] = {};
    last_targets['spell'] = {};

    last_sources['name'] = {};
    last_sources['time'] = {};

    SoundInfo['file'] = {"blade.mp3","matrix.mp3","mortalk.mp3","piratas.mp3",
                        "prepare.mp3",
                        "ious.mp3",
                        "finishHim.mp3",
                        "flaw.mp3",
                        "unreal.mp3",
                        "2kill.mp3","triplekill.mp3","multikill.mp3",
                        "firstblood.mp3","rampage.mp3","killingspree.mp3","dominating.mp3","MegaKill.mp3","unstoppable.mp3","wickedsick.mp3","monsterkill.mp3","ludicrouskill.mp3","godlike.mp3","holyshit.mp3"};
    SoundInfo['length'] = {41.7,26.3,52.0,36.0,
                            1.1,
                            3.0,
                            1.1,
                            1.6,
                            1.2,
                            1.5,1.7,1.9,
                            1.2,1.7,1.5,1.6,2.5,1.6,2.0,4.5,4.2,1.5,3.5};
    SoundInfo['type'] = {1,1,1,1, -- MUSIC
                        2,      -- ENEMY SELECTION
                        3,      -- PLAYER DIED
                        4,      -- ENEMY LOW HEALTH
                        5,      -- FLAWLESS VICTORY
                        6,      -- CRITICAL KILL
                        7,7,7,  -- MULTI KILLS
                        8,8,8,8,8,8,8,8,8,8,8}; -- KILL COUNT

    -- Sound Catalog stores the index of the first sound of a type and the size of that type
    SoundCatalog['size'] = {};
    SoundCatalog['first'] = {};
    local size = table.getn(SoundInfo['file']);
    local stype = 1;
    SoundCatalog['first'][1] = 1;
    for i = 1, size, 1 do
        if(SoundCatalog['size'][SoundInfo['type'][i]] == nil) then
            SoundCatalog['size'][SoundInfo['type'][i]] = 1;
        else
            SoundCatalog['size'][SoundInfo['type'][i]] = SoundCatalog['size'][SoundInfo['type'][i]] + 1;
        end
        if(stype ~= SoundInfo['type'][i]) then
            stype = SoundInfo['type'][i];
            SoundCatalog['first'][stype] = i;
        end
    end

    SlashCmdList["dotakillsounduu"] = DotaKillSoundUU_cmdHandler;
    SLASH_dotakillsounduu1 = "/dotakillsounduu";
    SLASH_dotakillsounduu2 = "/dks";
end
-------------------------------------------------------------------------------------
function DotaKillSoundUU_OnEvent(self,event,...)
    if (event == "UNIT_HEALTH") then
        UnitHealthChanged(...);
    elseif (event == "PLAYER_DEAD") then
        PlayerDead();
    elseif (event == "PLAYER_TARGET_CHANGED") then
        PlayerTargetChanged(...);
    elseif (event == "PLAYER_REGEN_ENABLED") then
        PlayerRegenEnabled();
    elseif (event == "PLAYER_REGEN_DISABLED") then
        PlayerRegenDisabled();
    elseif (event == "UNIT_COMBAT") then
        UnitCombat(...);
    elseif (event == "COMBAT_LOG_EVENT_UNFILTERED") then
        HandleCombatLog(...);
    end
end
-------------------------------------------------------------------------------------
function DotaKillSoundUU_OnUpdate(self,elapsed)
    if(sound_queue[head_sound] == nil) then return; end
    TimeElapsed_SinceUpdate = TimeElapsed_SinceUpdate + elapsed;
    if(TimeElapsed_SinceUpdate > UpdateInterval) then
        remainingTime_CurrentSound = remainingTime_CurrentSound - TimeElapsed_SinceUpdate;
        if(remainingTime_CurrentSound <= 0) then
            remainingTime_CurrentSound = DequeueFirstSoundAndPlay();
        end
        TimeElapsed_SinceUpdate = 0;
    end
end
-------------------------------------------------------------------------------------
function DotaKillSoundUU_cmdHandler(cmd)
    if (cmd == "focus") then
        if(DotaKillSoundUU_conf_focus == "ENABLED") then
            DotaKillSoundUU_conf_focus = "DISABLED";
        else
            DotaKillSoundUU_conf_focus = "ENABLED";
        end
        DEFAULT_CHAT_FRAME:AddMessage("DotaKillSoundUU: Cry on Focus " .. DotaKillSoundUU_conf_focus);
    elseif(cmd == "debug_sound") then
        DEFAULT_CHAT_FRAME:AddMessage("Sounds Pending (head:" .. head_sound .. " tail:" .. tail_sound .. "):");
        for i = 0, SOUNDBUFFER, 1 do
            if(sound_queue[i] ~= nil) then
                DEFAULT_CHAT_FRAME:AddMessage(i .. " * " .. sound_queue[i]);
            end
        end
    else
        DEFAULT_CHAT_FRAME:AddMessage("DotaKillSoundUU: <commands>");
        DEFAULT_CHAT_FRAME:AddMessage("   /dks focus : Toggles Cry on Focus ON/OFF");
    end
end

function UnitHealthChanged(...)
    -- if warning sound already sent
    if(finishHim) then return; end
		local arg1 = ...
    -- if it is the targeted enemy pvp player
    if(arg1 == "target" and UnitIsPVP("target") and UnitIsEnemy("player", "target") and UnitIsPlayer("target")) then
        -- if health dropped below 20% and we're alive
        if(UnitHealth("target") < UnitHealthMax("target")/5 and UnitHealth("target") > 0 and UnitHealth("player") > 0) then
            PlaySoundByType(4,0);
            --PlaySoundFile(soundPath .. "finishHim.mp3");
            finishHim = true;
        end
    end
end
-------------------------------------------------------------------------------------

function PlayerDead()
    local channel = nil;
    if IsInGroup(LE_PARTY_CATEGORY_INSTANCE) then
    	channel = "INSTANCE_CHAT"
    elseif IsInRaid() then
    	channel = "RAID"
    elseif IsInGroup() then
    	channel = "PARTY"
    elseif IsInGuild() then
    	channel = "GUILD"
    end

    if(channel ~= nil or DotaKillSoundUU_conf_focus == "DISABLED") then
        local sources = "";
        local source = "";
        local current_time = 0;
        for i = 0, BUFFER, 1 do
            if(last_sources['time'][i] ~= nil) then current_time = max(current_time, last_sources['time'][i]); end
        end
        local count = 0;
        for i = 0, BUFFER, 1 do
            if(last_sources['time'][i] ~= nil and last_sources['time'][i] > (current_time - 3)) then
                if(count==1) then
                    sources = sources .. source;
                elseif(count > 1) then
                    sources = sources .. ", " .. source;
                end
                source = last_sources['name'][i];
                count = count + 1;
            end
        end
        sources = sources .. " and " .. source;

        if(count > 2 and last_deathtime < current_time) then
            last_deathtime = current_time;
            if(DotaKillSoundUU_conf_focus=="ENABLED") then SendChatMessage("WHOOOOAAA! " .. sources .. " have just focused me. NOT FAIR >=(", channel);
            else DEFAULT_CHAT_FRAME:AddMessage("DKS: You were focused by: " .. sources, .8, .1, .1);
            end
        end
    end

    PlaySoundByType(3,0);
    --PlaySoundFile(soundPath .. "ious.mp3");
    DotaKillSoundUU_killCount = 0;
end
-------------------------------------------------------------------------------------

function PlayerTargetChanged()
    finishHim = false;
    
    -- if the target is enemy pvp player
    if(UnitIsPVP("target") and UnitIsEnemy("player", "target") and UnitIsPlayer("target")) then
        -- if health dropped below 20% and we're alive
        if(UnitHealth("target") < UnitHealthMax("target")/5 and UnitHealth("target") > 0 and UnitHealth("player") > 0) then
            PlaySoundByType(4,0);
            --PlaySoundFile(soundPath .. "finishHim.mp3");
            finishHim = true;
        -- else if both are alive
        elseif(UnitHealth("target") > 0 and UnitHealth("player") > 0) then
            PlaySoundByType(2,0);
            --PlaySoundFile(soundPath .. "prepare.mp3");
        end
    end
end
-------------------------------------------------------------------------------------

function PlayerRegenEnabled()
    --inCombat = false;
end
-------------------------------------------------------------------------------------

function PlayerRegenDisabled()
    flaw = true;
    if (UnitName("target") and UnitIsPlayer("target") and UnitIsEnemy("player", "target") and
        UnitHealth("target") > 0) then
        finishHim = false;
        --inCombat = true;
        if (GetTime() - lastMusicTime > MusicPlayTime ) then
            --music = math.random(4);
            MusicPlayTime = PlaySoundByType(1,0);
            --[[
            if (music == 1) then
                PlaySoundFile(soundPath .. "mortalk.mp3");
                MusicPlayTime = 53;
            elseif (music == 2) then
                PlaySoundFile(soundPath .. "piratas.mp3");
                MusicPlayTime = 36;
	    elseif (music == 3) then
                PlaySoundFile(soundPath .. "blade.mp3");
                MusicPlayTime = 42;
            elseif (music == 4) then
                PlaySoundFile(soundPath .. "matrix.mp3");
                MusicPlayTime = 27;
            end
            --]]
            lastMusicTime = GetTime();
        end
    end
end
-------------------------------------------------------------------------------------

function UnitCombat(...)
	local arg1,_,_,arg4 = ...
    if(arg1 == "player" and arg4 > 0) then
        flaw = false;
    end
end
-------------------------------------------------------------------------------------

-- /* If known target, update. Else if free slot, use it. Else overwrite oldest entry */
function add_target(newname, newtime, newhit, newcrit, newspell)
    local freeslot = -1;
    local oldest_entry = 0;
    local oldest_time = last_targets['time'][0];

    for i = 0, BUFFER, 1 do
        if(last_targets['name'][i] == newname) then
            last_targets['time'][i] = newtime;
            last_targets['hit'][i] = newhit;
            last_targets['crit'][i] = newcrit;
            last_targets['spell'][i] = newspell;
            return;
        end

        if(freeslot == -1) then
            if(last_targets['name'][i] == nil) then
                freeslot = i;
            elseif(oldest_time == nil or oldest_time > last_targets['time'][i]) then
                oldest_entry = i;
                oldest_time = last_targets['time'][i];
            end
        end
    end

    if(freeslot > -1) then
        last_targets['name'][freeslot] = newname;
        last_targets['time'][freeslot] = newtime;
        last_targets['hit'][freeslot] = newhit;
        last_targets['crit'][freeslot] = newcrit;
        last_targets['spell'][freeslot] = newspell;
    else
        last_targets['name'][oldest_entry] = newname;
        last_targets['time'][oldest_entry] = newtime;
        last_targets['hit'][oldest_entry] = newhit;
        last_targets['crit'][oldest_entry] = newcrit;
        last_targets['spell'][oldest_entry] = newspell;
    end
end
-------------------------------------------------
function remove_target(tar)
    for i = 0, BUFFER, 1 do
        if(last_targets['name'][i] == tar) then
            last_targets['name'][i] = nil;
            last_targets['time'][i] = nil;
            last_targets['hit'][i] = nil;
            last_targets['crit'][i] = nil;
            last_targets['spell'][i] = nil;
            break;
        end
    end
end
-------------------------------------------------
function check_target(tar)
    for i = 0, BUFFER, 1 do
        if(last_targets['name'][i] == tar) then
            YouKill(i);
            break;
        end
    end
end
-------------------------------------------------
-- /* If known source, update. Else if free slot, use it. Else overwrite oldest entry */
function add_source(newtime, newname)
    local freeslot = -1;
    local oldest_entry = 0;
    local oldest_time = last_sources['time'][0];

    for i = 0, BUFFER, 1 do
        if(last_sources['name'][i] == newname) then
            last_sources['time'][i] = newtime;
            return;
        end

        if(freeslot == -1) then
            if(last_sources['name'][i] == nil) then
                freeslot = i;
            elseif(oldest_time == nil or oldest_time > last_sources['time'][i]) then
                oldest_entry = i;
                oldest_time = last_sources['time'][i];
            end
        end
    end

    if(freeslot > -1) then
        last_sources['name'][freeslot] = newname;
        last_sources['time'][freeslot] = newtime;
    else
        last_sources['name'][oldest_entry] = newname;
        last_sources['time'][oldest_entry] = newtime;
    end
end
-------------------------------------------------
-- (1) timestamp,subevent,hideCaster,sourceGUID,sourceName,sourceFlags,sourceRaidFlags,destGUID,destName,destFlags,destRaidFlags (11)
-- SWING_DAMAGE: (12) amount,overkill,school,resisted,blocked,absorbed,critical (1 or nil),glancing (1 or nil),crushing (1 or nil) (20)
-- RANGE|SPELL_DAMAGE: (12) spellId,spellName,spellSchool,amount,overkill,school,resisted,blocked,absorbed,critical (1 or nil),glancing (1 or nil),crushing (1 or nil) (23)
function HandleCombatLog(...)
		local arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10,arg11,arg12,arg13,arg14,arg15,arg16,arg17,arg18,arg19,arg20,arg21,arg22,arg23 = ...
    
    local lastHit = 0;
    local lastCrit = 0;
    local lastSpell = nil;

    local timestamp = arg1;
    local event = arg2;

    local source_guid = arg4;
    local source = arg5;

    local victim_guid = arg8;
    local victim = arg9;

    if(source == player_name) then -- we do damage to ...
        if(victim ~= nil and victim_guid ~= nil and bit.band(strsub(victim_guid,1,5),"0x00F") == 0) then -- ... a player
            if(event == "SWING_DAMAGE") then
                lastHit = arg12;
                if(arg18 == 1) then -- 18 = melee crit
                    lastCrit = 1;
                end
                add_target(victim, timestamp, lastHit, lastCrit, lastSpell);
            elseif(event == "RANGE_DAMAGE" or event == "SPELL_DAMAGE" or event == "SPELL_PERIODIC_DAMAGE") then
                lastHit = arg15;
                if(arg21 == 1) then -- 21 = spell/periodic/range crit
                    lastCrit = 2;
                    lastSpell = arg13;
                end
                add_target(victim, timestamp, lastHit, lastCrit, lastSpell);
            end
        end
    elseif(victim ~= nil) then -- a victim received damage or died
        if(event == "SWING_DAMAGE" or event == "RANGE_DAMAGE" or event == "SPELL_DAMAGE" or event == "SPELL_PERIODIC_DAMAGE" or event == "ENVIRONMENTAL_DAMAGE") then
            if(victim ~= player_name) then -- we aren't the victim, someone else is making a frag
                remove_target(victim);
            elseif(event ~= "ENVIRONMENTAL_DAMAGE" and source ~= nil and source_guid ~= nil and bit.band(strsub(source_guid,1,5),"0x00F") == 0) then
                -- we're the victim, a player is attacking us
                add_source(timestamp,source);
            end
        elseif(event == "UNIT_DIED" and victim ~= player_name and victim_guid ~= nil and bit.band(strsub(victim_guid,1,5),"0x00F") == 0) then
            -- a player different from us died
            check_target(victim);
        end
    end
end

-------------------------------------------------------------------------------------
-- **** TODO: queue the sound files so if multiple awards are present reproduce them all in order ****
-- Beta implementation of the sound queue is present
function YouKill(index)
    DotaKillSoundUU_killCount = DotaKillSoundUU_killCount + 1;

    if (DotaKillSoundUU_record < DotaKillSoundUU_killCount) then
        DotaKillSoundUU_record = DotaKillSoundUU_killCount;
    end

    if (flaw) then
        -- PlaySoundFile( soundPath .. "flaw.mp3" );
        QueueSound(5,0);
        SendChatMessage("kill " .. last_targets['name'][index] .. " *PERFECT VICTORY* humiliation", "EMOTE");
    end

    if(last_targets['crit'][index] > 0) then
        if(last_targets['crit'][index] == 1) then
            SendChatMessage("has just ended " .. last_targets['name'][index] .. " with " .. last_targets['hit'][index] .. " critical strike!", "EMOTE");
        elseif(last_targets['crit'][index] == 2) then
            SendChatMessage("has just ended " .. last_targets['name'][index] .. " with " .. last_targets['hit'][index] .. " " .. last_targets['spell'][index] .. " critical!", "EMOTE");
        end
        -- if (not flaw) then PlaySoundFile( soundPath .. "unreal.mp3" ); end
        QueueSound(6,0);
    else
        SendChatMessage("pwned " .. last_targets['name'][index] .. "'s head!", "EMOTE");
    end

    if(GetTime() - lastKillTime < killTime) then
        multiKill = multiKill + 1;
        if(multiKill == 1) then
            -- if(last_targets['crit'][index] == 0 and not flaw) then PlaySoundFile(soundPath .. "2kill.mp3"); end
            QueueSound(7,1);
            SendChatMessage("just got a Doble Kill!!", "EMOTE");
        elseif(multiKill == 2) then
            -- if (last_targets['crit'][index] == 0 and not flaw) then PlaySoundFile(soundPath .. "triplekill.mp3"); end
            QueueSound(7,2);
            SendChatMessage("just got a Triple Kill!!", "EMOTE");
        elseif(multiKill > 2) then
            -- if (last_targets['crit'][index] == 0 and not flaw) then PlaySoundFile(soundPath .. "multikill.mp3"); end
            QueueSound(7,3);
            SendChatMessage("just got a Multi Kill!!", "EMOTE");
        end
    else
        multiKill = 0;
    end

    -- local cond = false;
    -- if(last_targets['crit'][index] == 0 and not flaw and multiKill == 0) then cond = true; end;

    if (DotaKillSoundUU_killCount == 1) then
        --if (cond) then PlaySoundFile(soundPath .. "firstblood.mp3"); end;
        QueueSound(8,1);
        SendChatMessage("just drew First Blood!", "EMOTE");
    elseif(DotaKillSoundUU_killCount == 2) then
        -- if (cond) then PlaySoundFile(soundPath .. "rampage.mp3"); end;
        QueueSound(8,2);
        SendChatMessage("is on a rampage!", "EMOTE");
    elseif(DotaKillSoundUU_killCount == 3) then
        -- if (cond) then PlaySoundFile(soundPath .. "killingspree.mp3"); end;
        QueueSound(8,3);
        SendChatMessage("is on a Killing spree!", "EMOTE");
    elseif(DotaKillSoundUU_killCount == 4) then
        -- if (cond) then PlaySoundFile(soundPath .. "dominating.mp3"); end;
        QueueSound(8,4);
        SendChatMessage("is dominating!", "EMOTE");
    elseif(DotaKillSoundUU_killCount == 5) then
        -- if (cond) then PlaySoundFile(soundPath .. "megaKill.mp3"); end;
        QueueSound(8,5);
        SendChatMessage("has a mega kill!", "EMOTE");
    elseif(DotaKillSoundUU_killCount == 6) then
        -- if (cond) then PlaySoundFile(soundPath .. "unstoppable.mp3"); end;
        QueueSound(8,6);
        SendChatMessage("is unstoppable!", "EMOTE");
    elseif(DotaKillSoundUU_killCount == 7) then
        -- if (cond) then PlaySoundFile(soundPath .. "wickedsick.mp3"); end;
        QueueSound(8,7);
        SendChatMessage("is wicked Sick!", "EMOTE");
    elseif(DotaKillSoundUU_killCount == 8) then
        -- if (cond) then PlaySoundFile(soundPath .. "monsterkill.mp3"); end;
        QueueSound(8,8);
        SendChatMessage("has a monster kill!", "EMOTE");
    elseif(DotaKillSoundUU_killCount == 9) then
        -- if (cond) then PlaySoundFile(soundPath .. "ludicrouskill.mp3"); end;
        QueueSound(8,9);
        SendChatMessage("has a ludicrous kill!", "EMOTE");
    elseif(DotaKillSoundUU_killCount == 10) then
        -- if (cond) then PlaySoundFile(soundPath .. "godlike.mp3"); end;
        QueueSound(8,10);
        SendChatMessage("is GODLIKE!", "EMOTE");
    else
        -- if (cond) then PlaySoundFile(soundPath .. "holyshit.mp3"); end;
        QueueSound(8,11);
        SendChatMessage("is beyond GODLIKE. Someone KILL HIM!!!", "EMOTE");
        SendChatMessage("has killed " .. DotaKillSoundUU_killCount .. " times in a row!", "EMOTE");
    end

    lastKillTime = GetTime();
    DEFAULT_CHAT_FRAME:AddMessage("***************KILLING BLOW!!!***************", 0, 0, 1);
end
-------------------------------------------------------------------------------------

-- Put a sound in queue, if buffer overflow delete oldest entry
function QueueSound(soundtype,soundnumber)
    tail_sound = (tail_sound+1)%SOUNDBUFFER;
    if(tail_sound == head_sound and sound_queue[head_sound] ~= nil) then
        head_sound = (head_sound+1)%SOUNDBUFFER;
    end
    sound_queue[tail_sound] = soundtype*100 + soundnumber;
end

-- Maybe we need to change some sounds in queue, for example if a killcount sound is in queue but new frag is awarded, the killcount sound should be changed to the new one.
-- Not implemented yet
function ChangeSound(soundfile1, soundfile2)
end

-- Extract the head sound in queue, reproduce it and return the sound length
function DequeueFirstSoundAndPlay()
    local SoundNumber = sound_queue[head_sound]%100;
    local SoundType = (sound_queue[head_sound] - SoundNumber)/100;
    sound_queue[head_sound] = nil;
    head_sound = (head_sound+1)%SOUNDBUFFER;
    return PlaySoundByType(SoundType,SoundNumber);
end

-- Play the sound especified by type and number and returns the sound length
function PlaySoundByType(soundtype,soundnumber)
    if(soundnumber == 0) then
        local music = math.random(SoundCatalog['size'][soundtype]);
        local index = SoundCatalog['first'][soundtype] + music - 1;
        PlaySoundFile(soundPath .. SoundInfo['file'][index]);
        return SoundInfo['length'][index];
    else
        local index = SoundCatalog['first'][soundtype] + soundnumber - 1;
        PlaySoundFile(soundPath .. SoundInfo['file'][index]);
        return SoundInfo['length'][index];
    end
end
